#include "MyHeap.h"


namespace GtpVisibilityPreprocessor {


/*****************************************************************************/
/*                         MyHeap implementation                             *
/*****************************************************************************/



inline void MyHeap::Place(Heapable *x, unsigned int i)
{
    mBuffer[i] = x;
    x->SetHeapPos(i);
}


void MyHeap::Swap(unsigned int i, unsigned int j)
{
    Heapable *tmp = mBuffer[i];

    Place(mBuffer[j], i);
    Place(tmp, j);
}


void MyHeap::UpHeap(unsigned int i)
{
    Heapable *moving = mBuffer[i];
    unsigned int index = i;
    unsigned int p = Parent(i);

    while ((index > 0) && (moving->SetPriority() > mBuffer[p]->SetPriority()))
    {
		Place(mBuffer[p], index);
		index = p;
		p = Parent(p);
    }

    if (index != i)
	{
		Place(moving, index);
	}
}


void MyHeap::DownHeap(unsigned int i)
{
    Heapable *moving = mBuffer[i];

    unsigned int index = i;
    unsigned int l = Left(i);
    unsigned int r = Right(i);
    unsigned int largest;

	while (l < (int)mBuffer.size())
	{
		if ((r < (unsigned int)mBuffer.size()) && (mBuffer[l]->SetPriority() < mBuffer[r]->SetPriority()))
			largest = r;
		else 
			largest = l;

		if (moving->SetPriority() < mBuffer[largest]->SetPriority())
		{
			Place(mBuffer[largest], index);
			index = largest;
			l = Left(index);
			r = Right(index);
		}
		else
		{
			break;
		}
	}

    if (index != i)
	{
		Place(moving, index);
	}
}


void MyHeap::Push(Heapable *t, float v)
{
    t->SetPriority(v);

	unsigned int i = (unsigned int)mBuffer.size();

	t->SetHeapPos(i);
	mBuffer.push_back(t);
    
    UpHeap(i);
}


bool MyHeap::Update(Heapable *t, float v)
{
    if (t->IsInHeap())
		return false;

    t->SetPriority(v);

    unsigned int i = t->GetHeapPos();

    if ((i > 0) && (v > mBuffer[Parent(i)]->SetPriority()))
		UpHeap(i);
    else
		DownHeap(i);

	return true;
}


Heapable *MyHeap::Pop()
{
    if (mBuffer.empty()) 
		return NULL;

    Swap(0, mBuffer.size() - 1);
    
	Heapable *dead = mBuffer.back();
	mBuffer.pop_back();

    DownHeap(0);
    dead->NotInHeap();

    return dead;
}


Heapable *MyHeap::Erase(Heapable *t)
{
    if (!t->IsInHeap()) 
		return NULL;

    int i = t->GetHeapPos();

    Swap(i, mBuffer.size() - 1);

	mBuffer.pop_back();
    t->NotInHeap();

    if (mBuffer[i]->SetPriority() < t->SetPriority())
		DownHeap(i);
	else
		UpHeap(i);

    return t;
}
}